//
// File:        testDEM.C
// Author:      Remigiusz Trzaska
// e-mail:      remek@npac.syr.edu
// Created:     Thu Dec 19 1996
// Copyright Remigiusz Trzaska 1996 NPAC, Syracuse University

// Description:
// 


#ifndef lint
static const char *sccsid = "@(#)testDEM.cc, Copyright NPAC Remigiusz Trzaska";
#endif

#include "Memory.h"
#include "wv.h"
#include <fstream.h>

float** data;
float maxVal=0.0;
float minVal=10000.0;

void
pgmSave(char* name, float** data, int dimx, int dimy)
{
   ofstream outFile(name);
   outFile << "P5" << endl << dimx << ' ' <<  dimy << endl << "255" << endl;
   for(int y=0; y<dimy; y++)
      for(int x=0; x<dimx; x++)
	 outFile.put((data[y][x]-minVal)/(maxVal-minVal)*255);
   outFile.close();
}


int main(int, char* argv[])
{
   ifstream inFile(argv[1]);
   
   int x;
   int y;
   
   int maxy=500;

   unsigned char _;
   for(int n=0; n<8; n++)
      inFile >> _;
   
   NewMatrix(data, maxy, maxy, float);

   for(y=0; y<maxy; y++) 
      for(x=0; x<maxy; x++)
      {
// 	 unsigned char msB;
	 unsigned char lsB;
// 	 inFile.get(msB);
	 inFile.get(lsB);
	 float tmp=/*msB*256.0+*/lsB;
	 data[y][x]=tmp;
	 if(tmp>maxVal) maxVal=tmp;
	 if(tmp<minVal) minVal=tmp;
      }
   
   cerr << "Data read..." << endl
	<< "The extremas were " << minVal << "," << maxVal << endl;
   inFile.close();
   
   pgmSave("original.pgm", data, 500, 500);

   ewv_t* compressor = new_ewv(500, 500, 0, 255, maxVal-minVal);
   unsigned char stream[4000000];
   int quality = atoi(argv[2]);
   int usedLen = 4000000;
   int retQ;

   if((retQ=enc_ewv(compressor, data[0], stream, &usedLen, quality))>=0)
   {
      cerr << "Used length: " << usedLen 
	   << " Quality returned: " << retQ << endl;
   }
   else
   {
      cerr << "Compression failed" << endl;
      exit(1);
   }

   for(int scale = 1; scale < 8; scale = 2*scale)
   {
      cerr << "Scale is " << scale << endl;

      float** datadec;
      NewMatrix(datadec, maxy/scale, maxy/scale, float);
      cerr << "after newmatrix\n";
      if(dec_ewv(compressor, stream, &usedLen, datadec[0], quality, scale)<0)
	 cerr << "Decompression failed" << endl;
      cerr << "Used length: " << usedLen << endl;

      char buf[256];
      sprintf(buf, "processed%d.pgm", scale);
      pgmSave(buf, datadec, 500/scale, 500/scale);
      cerr << "after processedsave\n";
      
      maxVal=0;
      minVal=10000;
      for(y=0; y<maxy/scale; y++) 
	 for(x=0; x<maxy/scale; x++)
	 {
	    float tmp=datadec[y][x];
	    if(tmp>maxVal) maxVal=tmp;
	    if(tmp<minVal) minVal=tmp;
	 }
      cerr << "The extremas were " << minVal << "," << maxVal << endl;
      
      sprintf(buf, "updated%d.pgm", scale);
      pgmSave(buf, datadec, 500/scale, 500/scale);
      delete datadec;
   }
   
      delete_ewv(&compressor);
}
